@@ -3,10 +3,11 @@ from copy import deepcopy
 from Crypto.PublicKey.RSA import RsaKey
 from datetime import datetime, timezone
 from redis.asyncio.client import Redis
-from typing import Dict, List, Optional
+from typing import Dict, List, Literal, Optional, Union, overload
 from uuid import UUID
 from maleo.soma.authorization import BearerAuth
 from maleo.soma.dtos.configurations.cache.redis import RedisCacheNamespaces
+from maleo.soma.enums.cardinality import Cardinality
 from maleo.soma.enums.environment import Environment
 from maleo.soma.enums.expiration import Expiration
 from maleo.soma.enums.logging import LogLevel
@@ -81,7 +82,6 @@ class BloodTypeClientService(MaleoClientService):
             redis_namespaces,
             service_context,
         )
-        # self._controllers = controllers
         self._namespace = self._redis_namespaces.create(
             self._key,
             RESOURCE.aggregate(),
@@ -96,8 +96,11 @@ class BloodTypeClientService(MaleoClientService):
             ),
         )
 
-    async def read_blood_types(
+    @overload
+    async def read(
         self,
+        cardinality: Literal[Cardinality.MULTIPLE],
+        *,
         operation_id: UUID,
         request_context: RequestContext,
         authentication: Authentication,
@@ -105,8 +108,33 @@ class BloodTypeClientService(MaleoClientService):
         headers: Optional[Dict[str, str]] = None,
     ) -> ReadMultipleResourceOperationResult[
         BloodTypeDataSchema, FlexiblePagination, None
+    ]: ...
+    @overload
+    async def read(
+        self,
+        cardinality: Literal[Cardinality.SINGLE],
+        *,
+        operation_id: UUID,
+        request_context: RequestContext,
+        authentication: Authentication,
+        parameters: ReadSingleParameter,
+        headers: Optional[Dict[str, str]] = None,
+    ) -> ReadSingleResourceOperationResult[BloodTypeDataSchema, None]: ...
+    async def read(
+        self,
+        cardinality: Cardinality,
+        *,
+        operation_id: UUID,
+        request_context: RequestContext,
+        authentication: Authentication,
+        parameters: Union[ReadMultipleParameter, ReadSingleParameter],
+        headers: Optional[Dict[str, str]] = None,
+    ) -> Union[
+        ReadMultipleResourceOperationResult[
+            BloodTypeDataSchema, FlexiblePagination, None
+        ],
+        ReadSingleResourceOperationResult[BloodTypeDataSchema, None],
     ]:
-        """Retrieve blood types from MaleoMetadata"""
         operation_action = ReadResourceOperationAction()
         executed_at = datetime.now(tz=timezone.utc)
 
@@ -115,7 +143,7 @@ class BloodTypeClientService(MaleoClientService):
         module, qualname = func.__module__, func.__qualname__
 
         # Define arguments being used in this function
-        positional_arguments = []
+        positional_arguments = [cardinality]
         keyword_arguments = {
             "authentication": authentication.model_dump(
                 mode="json",
@@ -150,40 +178,55 @@ class BloodTypeClientService(MaleoClientService):
 
             if result_str is not None:
                 completed_at = datetime.now(tz=timezone.utc)
-                result = ReadMultipleResourceOperationResult[
-                    BloodTypeDataSchema, FlexiblePagination, None
-                ].model_validate(json.loads(result_str))
-                ReadMultipleResourceOperationSchema[
-                    BloodTypeDataSchema, FlexiblePagination, None
-                ](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved multiple blood types from cache",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(
-                    self._logger, level=LogLevel.INFO
-                )
+
+                if isinstance(parameters, ReadMultipleParameter):
+                    result = ReadMultipleResourceOperationResult[
+                        BloodTypeDataSchema, FlexiblePagination, None
+                    ].model_validate(json.loads(result_str))
+                    ReadMultipleResourceOperationSchema[
+                        BloodTypeDataSchema, FlexiblePagination, None
+                    ](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved multiple blood types from cache",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(
+                        self._logger, LogLevel.INFO
+                    )
+                elif isinstance(parameters, ReadSingleParameter):
+                    result = ReadSingleResourceOperationResult[
+                        BloodTypeDataSchema, None
+                    ].model_validate(json.loads(result_str))
+                    ReadSingleResourceOperationSchema[BloodTypeDataSchema, None](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved single blood type from cache",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(self._logger, LogLevel.INFO)
                 return result
 
         operation_context = deepcopy(self._default_operation_context)
         async with self._http_client_manager.get() as http_client:
-            url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/"
-
-            # Parse parameters to query params
-            params = ReadMultipleQueryParameter.model_validate(
-                parameters.model_dump()
-            ).model_dump(exclude={"sort_columns", "date_filters"}, exclude_none=True)
-
             # Create headers
             base_headers = {
                 "Content-Type": "application/json",
@@ -207,6 +250,25 @@ class BloodTypeClientService(MaleoClientService):
 
             auth = BearerAuth(token) if token is not None else None
 
+            if isinstance(parameters, ReadMultipleParameter):
+                # Define URL
+                url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/"
+
+                # Parse parameters to query params
+                params = ReadMultipleQueryParameter.model_validate(
+                    parameters.model_dump()
+                ).model_dump(
+                    exclude={"sort_columns", "date_filters"}, exclude_none=True
+                )
+            elif isinstance(parameters, ReadSingleParameter):
+                # Define URL
+                url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/{parameters.identifier}/{parameters.value}"
+
+                # Parse parameters to query params
+                params = ReadSingleQueryParameterSchema.model_validate(
+                    parameters.model_dump()
+                ).model_dump(exclude_none=True)
+
             # Send request and wait for response
             response = await http_client.get(
                 url=url, params=params, headers=headers, auth=auth
@@ -214,207 +276,79 @@ class BloodTypeClientService(MaleoClientService):
 
             if response.is_success:
                 completed_at = datetime.now(tz=timezone.utc)
-                validated_response = MultipleDataResponseSchema[
-                    BloodTypeDataSchema, FlexiblePagination, None
-                ].model_validate(response.json())
-                data = DataPair[List[BloodTypeDataSchema], None](
-                    old=validated_response.data,
-                    new=None,
-                )
-                result = ReadMultipleResourceOperationResult[
-                    BloodTypeDataSchema, FlexiblePagination, None
-                ](
-                    data=data,
-                    pagination=validated_response.pagination,
-                    metadata=None,
-                    other=None,
-                )
-                ReadMultipleResourceOperationSchema[
-                    BloodTypeDataSchema, FlexiblePagination, None
-                ](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved multiple blood types from http request",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(
-                    self._logger, level=LogLevel.INFO
-                )
+
+                if isinstance(parameters, ReadMultipleParameter):
+                    validated_response = MultipleDataResponseSchema[
+                        BloodTypeDataSchema, FlexiblePagination, None
+                    ].model_validate(response.json())
+                    data = DataPair[List[BloodTypeDataSchema], None](
+                        old=validated_response.data,
+                        new=None,
+                    )
+                    result = ReadMultipleResourceOperationResult[
+                        BloodTypeDataSchema, FlexiblePagination, None
+                    ](
+                        data=data,
+                        pagination=validated_response.pagination,
+                        metadata=None,
+                        other=None,
+                    )
+                    ReadMultipleResourceOperationSchema[
+                        BloodTypeDataSchema, FlexiblePagination, None
+                    ](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved multiple blood types from http request",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(
+                        self._logger, level=LogLevel.INFO
+                    )
+                elif isinstance(parameters, ReadSingleParameter):
+                    validated_response = SingleDataResponseSchema[
+                        BloodTypeDataSchema, None
+                    ].model_validate(response.json())
+                    data = DataPair[BloodTypeDataSchema, None](
+                        old=validated_response.data,
+                        new=None,
+                    )
+                    result = ReadSingleResourceOperationResult[
+                        BloodTypeDataSchema, None
+                    ](
+                        data=data,
+                        pagination=validated_response.pagination,
+                        metadata=None,
+                        other=None,
+                    )
+                    ReadSingleResourceOperationSchema[BloodTypeDataSchema, None](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved single blood type from http request",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(self._logger, level=LogLevel.INFO)
 
                 if parameters.use_cache:
                     await self._redis.set(
-                        cache_key, result.model_dump_json(), Expiration.EXP_1WK.value
-                    )
-
-                return result
-
-            self._raise_resource_http_request_error(
-                response=response,
-                operation_id=operation_id,
-                operation_context=operation_context,
-                executed_at=executed_at,
-                operation_action=operation_action,
-                request_context=request_context,
-                authentication=authentication,
-                resource=RESOURCE,
-            )
-
-    async def read_blood_type(
-        self,
-        operation_id: UUID,
-        request_context: RequestContext,
-        authentication: Authentication,
-        parameters: ReadSingleParameter,
-        headers: Optional[Dict[str, str]] = None,
-    ) -> ReadSingleResourceOperationResult[BloodTypeDataSchema, None]:
-        """Retrieve blood type from MaleoMetadata"""
-        operation_action = ReadResourceOperationAction()
-        executed_at = datetime.now(tz=timezone.utc)
-
-        # Get function identifier
-        func = self.__class__
-        module, qualname = func.__module__, func.__qualname__
-
-        # Define arguments being used in this function
-        positional_arguments = []
-        keyword_arguments = {
-            "authentication": authentication.model_dump(
-                mode="json",
-                exclude={
-                    "credentials": {
-                        "token": {
-                            "payload": {
-                                "iat_dt",
-                                "iat",
-                                "exp_dt",
-                                "exp",
-                            }
-                        }
-                    }
-                },
-            ),
-            "parameters": parameters.model_dump(mode="json"),
-        }
-
-        # Define full function string
-        function = f"{qualname}({json.dumps(positional_arguments)}|{json.dumps(keyword_arguments)})"
-
-        # Define full cache key
-        cache_key = build_key(module, function, namespace=self._namespace)
-
-        if parameters.use_cache:
-            # Cache operation
-            operation_context = deepcopy(self._default_operation_context)
-            operation_context.target.type = OperationTarget.CACHE
-            # Check redis for data
-            result_str = await self._redis.get(cache_key)
-
-            if result_str is not None:
-                completed_at = datetime.now(tz=timezone.utc)
-                result = ReadSingleResourceOperationResult[
-                    BloodTypeDataSchema, None
-                ].model_validate(json.loads(result_str))
-                ReadSingleResourceOperationSchema[BloodTypeDataSchema, None](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved single blood type from cache",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(self._logger, level=LogLevel.INFO)
-                return result
-
-        # HTTP Request
-        operation_context = deepcopy(self._default_operation_context)
-        async with self._http_client_manager.get() as client:
-            # Define URL
-            url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/{parameters.identifier}/{parameters.value}"
-
-            # Parse parameters to query params
-            params = ReadSingleQueryParameterSchema.model_validate(
-                parameters.model_dump()
-            ).model_dump(exclude_none=True)
-
-            # Create headers
-            base_headers = {
-                "Content-Type": "application/json",
-                "X-Operation_Id": str(operation_id),
-            }
-            if headers is not None:
-                headers = merge_dicts(base_headers, headers)
-            else:
-                headers = base_headers
-
-            # Create auth
-            token = None
-            if authentication.credentials.token is not None:
-                try:
-                    token = reencode(
-                        payload=authentication.credentials.token.payload,
-                        key=self._private_key,
-                    )
-                except Exception:
-                    pass
-
-            auth = BearerAuth(token) if token is not None else None
-
-            # Send request and wait for response
-            response = await client.get(
-                url=url, params=params, headers=headers, auth=auth
-            )
-
-            if response.is_success:
-                completed_at = datetime.now(tz=timezone.utc)
-                validated_response = SingleDataResponseSchema[
-                    BloodTypeDataSchema, None
-                ].model_validate(response.json())
-                data = DataPair[BloodTypeDataSchema, None](
-                    old=validated_response.data,
-                    new=None,
-                )
-                result = ReadSingleResourceOperationResult[BloodTypeDataSchema, None](
-                    data=data,
-                    pagination=validated_response.pagination,
-                    metadata=None,
-                    other=None,
-                )
-                ReadSingleResourceOperationSchema[BloodTypeDataSchema, None](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved single blood type from http request",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(self._logger, level=LogLevel.INFO)
-
-                if parameters.use_cache:
-                    await self._redis.set(
-                        cache_key, result.model_dump_json(), Expiration.EXP_1WK.value
+                        cache_key, result.model_dump_json(), Expiration.EXP_1MO.value
                     )
 
                 return result
