@@ -3,10 +3,11 @@ from copy import deepcopy
 from Crypto.PublicKey.RSA import RsaKey
 from datetime import datetime, timezone
 from redis.asyncio.client import Redis
-from typing import Dict, List, Optional
+from typing import Dict, List, Literal, Optional, Union, overload
 from uuid import UUID
 from maleo.soma.authorization import BearerAuth
 from maleo.soma.dtos.configurations.cache.redis import RedisCacheNamespaces
+from maleo.soma.enums.cardinality import Cardinality
 from maleo.soma.enums.environment import Environment
 from maleo.soma.enums.expiration import Expiration
 from maleo.soma.enums.logging import LogLevel
@@ -15,7 +16,6 @@ from maleo.soma.managers.client.maleo import MaleoClientService
 from maleo.soma.managers.client.http import HTTPClientManager
 from maleo.soma.managers.credential import CredentialManager
 from maleo.soma.schemas.authentication import Authentication
-from maleo.soma.schemas.authorization import Authorization
 from maleo.soma.schemas.data import DataPair
 from maleo.soma.schemas.operation.context import (
     OperationContextSchema,
@@ -84,7 +84,6 @@ class OrganizationTypeClientService(MaleoClientService):
             redis_namespaces,
             service_context,
         )
-        # self._controllers = controllers
         self._namespace = self._redis_namespaces.create(
             self._key,
             RESOURCE.aggregate(),
@@ -99,18 +98,45 @@ class OrganizationTypeClientService(MaleoClientService):
             ),
         )
 
-    async def read_organization_types(
+    @overload
+    async def read(
         self,
+        cardinality: Literal[Cardinality.MULTIPLE],
+        *,
         operation_id: UUID,
         request_context: RequestContext,
         authentication: Authentication,
         parameters: ReadMultipleParameter,
-        authorization: Optional[Authorization] = None,
         headers: Optional[Dict[str, str]] = None,
     ) -> ReadMultipleResourceOperationResult[
         OrganizationTypeDataSchema, StrictPagination, None
+    ]: ...
+    @overload
+    async def read(
+        self,
+        cardinality: Literal[Cardinality.SINGLE],
+        *,
+        operation_id: UUID,
+        request_context: RequestContext,
+        authentication: Authentication,
+        parameters: ReadSingleParameter,
+        headers: Optional[Dict[str, str]] = None,
+    ) -> ReadSingleResourceOperationResult[OrganizationTypeDataSchema, None]: ...
+    async def read(
+        self,
+        cardinality: Cardinality,
+        *,
+        operation_id: UUID,
+        request_context: RequestContext,
+        authentication: Authentication,
+        parameters: Union[ReadMultipleParameter, ReadSingleParameter],
+        headers: Optional[Dict[str, str]] = None,
+    ) -> Union[
+        ReadMultipleResourceOperationResult[
+            OrganizationTypeDataSchema, StrictPagination, None
+        ],
+        ReadSingleResourceOperationResult[OrganizationTypeDataSchema, None],
     ]:
-        """Retrieve organization types from MaleoMetadata"""
         operation_action = ReadResourceOperationAction()
         executed_at = datetime.now(tz=timezone.utc)
 
@@ -119,7 +145,7 @@ class OrganizationTypeClientService(MaleoClientService):
         module, qualname = func.__module__, func.__qualname__
 
         # Define arguments being used in this function
-        positional_arguments = []
+        positional_arguments = [cardinality]
         keyword_arguments = {
             "authentication": authentication.model_dump(
                 mode="json",
@@ -154,40 +180,55 @@ class OrganizationTypeClientService(MaleoClientService):
 
             if result_str is not None:
                 completed_at = datetime.now(tz=timezone.utc)
-                result = ReadMultipleResourceOperationResult[
-                    OrganizationTypeDataSchema, StrictPagination, None
-                ].model_validate(json.loads(result_str))
-                ReadMultipleResourceOperationSchema[
-                    OrganizationTypeDataSchema, StrictPagination, None
-                ](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved multiple organization types from cache",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(
-                    self._logger, level=LogLevel.INFO
-                )
+
+                if isinstance(parameters, ReadMultipleParameter):
+                    result = ReadMultipleResourceOperationResult[
+                        OrganizationTypeDataSchema, StrictPagination, None
+                    ].model_validate(json.loads(result_str))
+                    ReadMultipleResourceOperationSchema[
+                        OrganizationTypeDataSchema, StrictPagination, None
+                    ](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved multiple organization types from cache",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(
+                        self._logger, LogLevel.INFO
+                    )
+                elif isinstance(parameters, ReadSingleParameter):
+                    result = ReadSingleResourceOperationResult[
+                        OrganizationTypeDataSchema, None
+                    ].model_validate(json.loads(result_str))
+                    ReadSingleResourceOperationSchema[OrganizationTypeDataSchema, None](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved single organization type from cache",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(self._logger, LogLevel.INFO)
                 return result
 
         operation_context = deepcopy(self._default_operation_context)
         async with self._http_client_manager.get() as http_client:
-            url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/"
-
-            # Parse parameters to query params
-            params = ReadMultipleQueryParameter.model_validate(
-                parameters.model_dump()
-            ).model_dump(exclude={"sort_columns", "date_filters"}, exclude_none=True)
-
             # Create headers
             base_headers = {
                 "Content-Type": "application/json",
@@ -209,18 +250,27 @@ class OrganizationTypeClientService(MaleoClientService):
                 except Exception:
                     pass
 
-            if (
-                token is None
-                and authorization is not None
-                and authorization.scheme == "Bearer"
-            ):
-                token = authorization.credentials
-
-            if token is None:
-                token = self._credential_manager.token
-
             auth = BearerAuth(token) if token is not None else None
 
+            if isinstance(parameters, ReadMultipleParameter):
+                # Define URL
+                url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/"
+
+                # Parse parameters to query params
+                params = ReadMultipleQueryParameter.model_validate(
+                    parameters.model_dump()
+                ).model_dump(
+                    exclude={"sort_columns", "date_filters"}, exclude_none=True
+                )
+            elif isinstance(parameters, ReadSingleParameter):
+                # Define URL
+                url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/{parameters.identifier}/{parameters.value}"
+
+                # Parse parameters to query params
+                params = ReadSingleQueryParameterSchema.model_validate(
+                    parameters.model_dump()
+                ).model_dump(exclude_none=True)
+
             # Send request and wait for response
             response = await http_client.get(
                 url=url, params=params, headers=headers, auth=auth
@@ -228,220 +278,79 @@ class OrganizationTypeClientService(MaleoClientService):
 
             if response.is_success:
                 completed_at = datetime.now(tz=timezone.utc)
-                validated_response = MultipleDataResponseSchema[
-                    OrganizationTypeDataSchema, StrictPagination, None
-                ].model_validate(response.json())
-                data = DataPair[List[OrganizationTypeDataSchema], None](
-                    old=validated_response.data,
-                    new=None,
-                )
-                result = ReadMultipleResourceOperationResult[
-                    OrganizationTypeDataSchema, StrictPagination, None
-                ](
-                    data=data,
-                    pagination=validated_response.pagination,
-                    metadata=None,
-                    other=None,
-                )
-                ReadMultipleResourceOperationSchema[
-                    OrganizationTypeDataSchema, StrictPagination, None
-                ](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved multiple organization types from http request",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(
-                    self._logger, level=LogLevel.INFO
-                )
+
+                if isinstance(parameters, ReadMultipleParameter):
+                    validated_response = MultipleDataResponseSchema[
+                        OrganizationTypeDataSchema, StrictPagination, None
+                    ].model_validate(response.json())
+                    data = DataPair[List[OrganizationTypeDataSchema], None](
+                        old=validated_response.data,
+                        new=None,
+                    )
+                    result = ReadMultipleResourceOperationResult[
+                        OrganizationTypeDataSchema, StrictPagination, None
+                    ](
+                        data=data,
+                        pagination=validated_response.pagination,
+                        metadata=None,
+                        other=None,
+                    )
+                    ReadMultipleResourceOperationSchema[
+                        OrganizationTypeDataSchema, StrictPagination, None
+                    ](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved multiple organization types from http request",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(
+                        self._logger, level=LogLevel.INFO
+                    )
+                elif isinstance(parameters, ReadSingleParameter):
+                    validated_response = SingleDataResponseSchema[
+                        OrganizationTypeDataSchema, None
+                    ].model_validate(response.json())
+                    data = DataPair[OrganizationTypeDataSchema, None](
+                        old=validated_response.data,
+                        new=None,
+                    )
+                    result = ReadSingleResourceOperationResult[
+                        OrganizationTypeDataSchema, None
+                    ](
+                        data=data,
+                        pagination=validated_response.pagination,
+                        metadata=None,
+                        other=None,
+                    )
+                    ReadSingleResourceOperationSchema[OrganizationTypeDataSchema, None](
+                        service_context=self._service_context,
+                        id=operation_id,
+                        context=operation_context,
+                        timestamp=OperationTimestamp(
+                            executed_at=executed_at,
+                            completed_at=completed_at,
+                            duration=(completed_at - executed_at).total_seconds(),
+                        ),
+                        summary="Successfully retrieved single organization type from http request",
+                        request_context=request_context,
+                        authentication=authentication,
+                        action=operation_action,
+                        resource=RESOURCE,
+                        result=result,
+                    ).log(self._logger, level=LogLevel.INFO)
 
                 if parameters.use_cache:
                     await self._redis.set(
-                        cache_key, result.model_dump_json(), Expiration.EXP_1WK.value
-                    )
-
-                return result
-
-            self._raise_resource_http_request_error(
-                response=response,
-                operation_id=operation_id,
-                operation_context=operation_context,
-                executed_at=executed_at,
-                operation_action=operation_action,
-                request_context=request_context,
-                authentication=authentication,
-                resource=RESOURCE,
-            )
-
-    async def read_organization_type(
-        self,
-        operation_id: UUID,
-        request_context: RequestContext,
-        authentication: Authentication,
-        parameters: ReadSingleParameter,
-        authorization: Optional[Authorization] = None,
-        headers: Optional[Dict[str, str]] = None,
-    ) -> ReadSingleResourceOperationResult[OrganizationTypeDataSchema, None]:
-        """Retrieve organization type from MaleoMetadata"""
-        operation_action = ReadResourceOperationAction()
-        executed_at = datetime.now(tz=timezone.utc)
-
-        # Get function identifier
-        func = self.__class__
-        module, qualname = func.__module__, func.__qualname__
-
-        # Define arguments being used in this function
-        positional_arguments = []
-        keyword_arguments = {
-            "authentication": authentication.model_dump(
-                mode="json",
-                exclude={
-                    "credentials": {
-                        "token": {
-                            "payload": {
-                                "iat_dt",
-                                "iat",
-                                "exp_dt",
-                                "exp",
-                            }
-                        }
-                    }
-                },
-            ),
-            "parameters": parameters.model_dump(mode="json"),
-        }
-
-        # Define full function string
-        function = f"{qualname}({json.dumps(positional_arguments)}|{json.dumps(keyword_arguments)})"
-
-        # Define full cache key
-        cache_key = build_key(module, function, namespace=self._namespace)
-
-        if parameters.use_cache:
-            # Cache operation
-            operation_context = deepcopy(self._default_operation_context)
-            operation_context.target.type = OperationTarget.CACHE
-            # Check redis for data
-            result_str = await self._redis.get(cache_key)
-
-            if result_str is not None:
-                completed_at = datetime.now(tz=timezone.utc)
-                result = ReadSingleResourceOperationResult[
-                    OrganizationTypeDataSchema, None
-                ].model_validate(json.loads(result_str))
-                ReadSingleResourceOperationSchema[OrganizationTypeDataSchema, None](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved single organization type from cache",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(self._logger, level=LogLevel.INFO)
-                return result
-
-        # HTTP Request
-        operation_context = deepcopy(self._default_operation_context)
-        async with self._http_client_manager.get() as client:
-            # Define URL
-            url = f"{self._url}/v1/{RESOURCE.identifiers[0].url_slug}/{parameters.identifier}/{parameters.value}"
-
-            # Parse parameters to query params
-            params = ReadSingleQueryParameterSchema.model_validate(
-                parameters.model_dump()
-            ).model_dump(exclude_none=True)
-
-            # Create headers
-            base_headers = {
-                "Content-Type": "application/json",
-                "X-Operation_Id": str(operation_id),
-            }
-            if headers is not None:
-                headers = merge_dicts(base_headers, headers)
-            else:
-                headers = base_headers
-
-            # Create auth
-            token = None
-            if authentication.credentials.token is not None:
-                try:
-                    token = reencode(
-                        payload=authentication.credentials.token.payload,
-                        key=self._private_key,
-                    )
-                except Exception:
-                    pass
-
-            if (
-                token is None
-                and authorization is not None
-                and authorization.scheme == "Bearer"
-            ):
-                token = authorization.credentials
-
-            if token is None:
-                token = self._credential_manager.token
-
-            auth = BearerAuth(token) if token is not None else None
-
-            # Send request and wait for response
-            response = await client.get(
-                url=url, params=params, headers=headers, auth=auth
-            )
-
-            if response.is_success:
-                completed_at = datetime.now(tz=timezone.utc)
-                validated_response = SingleDataResponseSchema[
-                    OrganizationTypeDataSchema, None
-                ].model_validate(response.json())
-                data = DataPair[OrganizationTypeDataSchema, None](
-                    old=validated_response.data,
-                    new=None,
-                )
-                result = ReadSingleResourceOperationResult[
-                    OrganizationTypeDataSchema, None
-                ](
-                    data=data,
-                    pagination=validated_response.pagination,
-                    metadata=None,
-                    other=None,
-                )
-                ReadSingleResourceOperationSchema[OrganizationTypeDataSchema, None](
-                    service_context=self._service_context,
-                    id=operation_id,
-                    context=operation_context,
-                    timestamp=OperationTimestamp(
-                        executed_at=executed_at,
-                        completed_at=completed_at,
-                        duration=(completed_at - executed_at).total_seconds(),
-                    ),
-                    summary="Successfully retrieved single organization type from http request",
-                    request_context=request_context,
-                    authentication=authentication,
-                    action=operation_action,
-                    resource=RESOURCE,
-                    result=result,
-                ).log(self._logger, level=LogLevel.INFO)
-
-                if parameters.use_cache:
-                    await self._redis.set(
-                        cache_key, result.model_dump_json(), Expiration.EXP_1WK.value
+                        cache_key, result.model_dump_json(), Expiration.EXP_1MO.value
                     )
 
                 return result
