@@ -18,7 +18,9 @@ IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
 
 RESOURCE = Resource(
     identifiers=[
-        ResourceIdentifier(key="blood_types", name="BloodTypes", url_slug="blood-types")
+        ResourceIdentifier(
+            key="blood_types", name="Blood Types", url_slug="blood-types"
+        )
     ],
     details=None,
 )
