@@ -20,7 +20,7 @@ IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
 RESOURCE = Resource(
     identifiers=[
         ResourceIdentifier(
-            key="medical_roles", name="MedicalRoled", url_slug="medical-roles"
+            key="medical_roles", name="Medical Roles", url_slug="medical-roles"
         )
     ],
     details=None,
