@@ -1,7 +1,7 @@
 from pydantic import BaseModel, Field
 from typing import List, Optional
 from maleo.soma.mixins.data import DataIdentifier, DataStatus, DataTimestamp
-from maleo.metadata.dtos.blood_type import BloodTypeDataDTO
+from maleo.metadata.dtos.data.blood_type import BloodTypeDataDTO
 from maleo.metadata.enums.blood_type import BloodType
 
 
