@@ -1,7 +1,7 @@
 from pydantic import BaseModel, Field
 from typing import List, Optional
 from maleo.soma.mixins.data import DataIdentifier, DataStatus, DataTimestamp
-from maleo.metadata.dtos.gender import GenderDataDTO
+from maleo.metadata.dtos.data.gender import GenderDataDTO
 from maleo.metadata.enums.gender import Gender
 
 
