@@ -1,7 +1,7 @@
 from pydantic import BaseModel, Field
 from typing import List, Optional
 from maleo.soma.mixins.data import DataIdentifier, DataStatus, DataTimestamp
-from maleo.metadata.dtos.medical_role import MedicalRoleDataDTO
+from maleo.metadata.dtos.data.medical_role import MedicalRoleDataDTO
 from maleo.metadata.enums.medical_role import MedicalRole
 
 
