@@ -1,7 +1,7 @@
 from pydantic import BaseModel, Field
 from typing import List, Optional
 from maleo.soma.mixins.data import DataIdentifier, DataStatus, DataTimestamp
-from maleo.metadata.dtos.organization_type import OrganizationTypeDataDTO
+from maleo.metadata.dtos.data.organization_type import OrganizationTypeDataDTO
 from maleo.metadata.enums.organization_type import OrganizationType
 
 
