@@ -1,7 +1,7 @@
 from pydantic import BaseModel, Field
 from typing import List, Optional
 from maleo.soma.mixins.data import DataIdentifier, DataStatus, DataTimestamp
-from maleo.metadata.dtos.service import ServiceDataDTO, FullServiceDataDTO
+from maleo.metadata.dtos.data.service import ServiceDataDTO, FullServiceDataDTO
 from maleo.metadata.enums.service import Service
 
 
