@@ -1,7 +1,7 @@
 from pydantic import BaseModel, Field
 from typing import List, Optional
 from maleo.soma.mixins.data import DataIdentifier, DataStatus, DataTimestamp
-from maleo.metadata.dtos.system_role import SystemRoleDataDTO
+from maleo.metadata.dtos.data.system_role import SystemRoleDataDTO
 from maleo.metadata.enums.system_role import SystemRole
 
 
