@@ -1,7 +1,7 @@
 from pydantic import BaseModel, Field
 from typing import List, Optional
 from maleo.soma.mixins.data import DataIdentifier, DataStatus, DataTimestamp
-from maleo.metadata.dtos.user_type import UserTypeDataDTO
+from maleo.metadata.dtos.data.user_type import UserTypeDataDTO
 from maleo.metadata.enums.user_type import UserType
 
 
